"""Top-level package for mdns-beacon."""
from .beacon import Beacon  # noqa: F401
from .listener import BeaconListener  # noqa: F401

__author__ = """Federico Jaureguialzo"""
__email__ = "fedejaure@gmail.com"
__version__ = "0.4.2"
